@@ -33,6 +33,35 @@
 #include "crypto.h"
 #include "nm-utils-private.h"
 
+/**
+ * SECTION:nm-setting-8021x
+ * @short_description: Describes 802.1x-authenticated connection properties
+ * @include: nm-setting-8021x.h
+ *
+ * The #NMSetting8021x object is a #NMSetting subclass that describes
+ * properties necessary for connection to 802.1x-authenticated networks, such as
+ * WPA and WPA2 Enterprise WiFi networks and wired 802.1x networks.  802.1x
+ * connections typically use certificates and/or EAP authentication methods to
+ * securely verify, identify, and authenticate the client to the network itself,
+ * instead of simply relying on a widely shared static key.
+ *
+ * It's a good idea to read up on wpa_supplicant configuration before using this
+ * setting extensively, since most of the options here correspond closely with
+ * the relevant wpa_supplicant configuration options.
+ *
+ * Furthermore, to get a good idea of 802.1x, EAP, TLS, TTLS, etc and their
+ * applications to WiFi and wired networks, you'll want to get copies of the
+ * following books.
+ *
+ *  802.11 Wireless Networks: The Definitive Guide, Second Edition
+ *       Author: Matthew Gast
+ *       ISBN: 978-0596100520
+ *
+ *  Cisco Wireless LAN Security
+ *       Authors: Krishna Sankar, Sri Sundaralingam, Darrin Miller, and Andrew Balinsky
+ *       ISBN: 978-1587051548
+ **/
+
 GQuark
 nm_setting_802_1x_error_quark (void)
 {
@@ -555,18 +584,18 @@ nm_setting_802_1x_set_private_key_from_file (NMSetting8021x *self,
 			priv->private_key_password = g_strdup (password);
 			if (out_ck_type)
 				*out_ck_type = NM_SETTING_802_1X_CK_TYPE_PKCS12;
+
+			/* As required by NM, set the client-cert property to the same PKCS#12 data */
+			if (priv->client_cert)
+				g_byte_array_free (priv->client_cert, TRUE);
+
+			priv->client_cert = g_byte_array_sized_new (priv->private_key->len);
+			g_byte_array_append (priv->client_cert, priv->private_key->data, priv->private_key->len);
 			break;
 		default:
 			g_assert_not_reached ();
 			break;
 		} 
-
-		/* As required by NM, set the client-cert property to the same PKCS#12 data */
-		if (priv->client_cert)
-			g_byte_array_free (priv->client_cert, TRUE);
-
-		priv->client_cert = g_byte_array_sized_new (priv->private_key->len);
-		g_byte_array_append (priv->client_cert, priv->private_key->data, priv->private_key->len);
 	} else {
 		/* As a special case for private keys, even if the decrypt fails,
 		 * return the key's file type.
@@ -649,18 +678,18 @@ nm_setting_802_1x_set_phase2_private_key_from_file (NMSetting8021x *self,
 			priv->phase2_private_key_password = g_strdup (password);
 			if (out_ck_type)
 				*out_ck_type = NM_SETTING_802_1X_CK_TYPE_PKCS12;
+
+			/* As required by NM, set the client-cert property to the same PKCS#12 data */
+			if (priv->phase2_client_cert)
+				g_byte_array_free (priv->phase2_client_cert, TRUE);
+
+			priv->phase2_client_cert = g_byte_array_sized_new (priv->phase2_private_key->len);
+			g_byte_array_append (priv->phase2_client_cert, priv->phase2_private_key->data, priv->phase2_private_key->len);
 			break;
 		default:
 			g_assert_not_reached ();
 			break;
 		} 
-
-		/* As required by NM, set the client-cert property to the same PKCS#12 data */
-		if (priv->phase2_client_cert)
-			g_byte_array_free (priv->phase2_client_cert, TRUE);
-
-		priv->phase2_client_cert = g_byte_array_sized_new (priv->phase2_private_key->len);
-		g_byte_array_append (priv->phase2_client_cert, priv->phase2_private_key->data, priv->phase2_private_key->len);
 	} else {
 		/* As a special case for private keys, even if the decrypt fails,
 		 * return the key's file type.
@@ -947,7 +976,7 @@ static EAPMethodsTable eap_methods_table[] = {
 	{ "peap", need_secrets_phase2, verify_ttls },
 	{ "ttls", need_secrets_phase2, verify_ttls },
 	{ "sim", need_secrets_sim, NULL },
-	{ "gtc", NULL, NULL },  // FIXME: implement
+	{ "gtc", need_secrets_password, verify_identity },
 	{ "otp", NULL, NULL },  // FIXME: implement
 	{ NULL, NULL, NULL }
 };
@@ -978,7 +1007,7 @@ need_secrets_phase2 (NMSetting8021x *self,
 	for (i = 0; eap_methods_table[i].method; i++) {
 		if (eap_methods_table[i].ns_func == NULL)
 			continue;
-		if (strcmp (eap_methods_table[i].method, method)) {
+		if (!strcmp (eap_methods_table[i].method, method)) {
 			(*eap_methods_table[i].ns_func) (self, secrets, TRUE);
 			break;
 		}
